/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.calendar.manager;

import java.io.IOException;

import java.util.Date;
import java.util.TimeZone;

import sync4j.foundation.pdi.utils.TimeUtils;

import sync4j.exchange.items.calendar.dao.CalendarDAO;
import sync4j.exchange.items.calendar.model.Calendar;
import sync4j.exchange.items.common.dao.ItemDAO;
import sync4j.exchange.items.common.model.Item;
import sync4j.exchange.DataAccessException;

import sync4j.exchange.util.DateTools;
import sync4j.exchange.util.StringTools;

import sync4j.exchange.xml.XmlParser;

/*
 * This class defines methods to access calendars data
 * in exchange server datastore
 *
 * @version $Id: CalendarManager.java,v 1.13 2006/02/13 13:39:36 fabius Exp $
 *
 **/
public class CalendarManager {

    //------------------------------------------------------------- Constants

    //------------------------------------------------------------- Private Data

    private CalendarDAO cd   = null ;
    private ItemDAO     id   = null ;

    // The timezone to use (if required) in the dates conversion
    private TimeZone timezone = null;

    //------------------------------------------------------------- Costructor

    public CalendarManager(String exchangeServerHost ,
                           int    exchangeServerPort ,
                           TimeZone timezone)
        throws DataAccessException {

        this.cd = new CalendarDAO (exchangeServerHost ,
                                   exchangeServerPort ,
                                   timezone);

        this.id = new ItemDAO (exchangeServerHost,
                               exchangeServerPort);

        this.timezone = timezone;
    }

    //------------------------------------------------------------- Public methods

    /**
     * get all calendars
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find calendars
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Calendar[] getAllCalendars(String   username    ,
                                      String   credentials ,
                                      String   exchangeFolder )
        throws DataAccessException {

        return this.cd.getCalendars(username      ,
                                    credentials   ,
                                    new String[0] ,
                                    exchangeFolder     );
    }


    /**
     * get calendar twin
     *
     * @param calendar the calendar object
     * @param principalId
     * @param sourceURI
     *
     * @return find contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Calendar getCalendarTwin(Calendar calendar      ,
                                    int      principalId   ,
                                    String   sourceURI)
    throws DataAccessException {


        long crc = StringTools.getCRCFromString(calendar.getDtStart().getPropertyValueAsString()  +
                                                calendar.getDtEnd().getPropertyValueAsString()    +
                                                calendar.getLocation().getPropertyValueAsString() );

        Item item = id.getTwin(crc           ,
                               principalId   ,
                               sourceURI);

        if (item == null) {
            return null;
        }

        return new Calendar(item.getId(), item.getLastModified());
    }

    /**
     * get all Exchange Server twin items
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     * @param ids
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item[] getTwinCalendarItems(String username       ,
                                       String credentials    ,
                                       String exchangeFolder ,
                                       String[] ids)
    throws DataAccessException {

        return id.getTwinCalendarItems(username    ,
                                       credentials ,
                                       exchangeFolder,
                                       timezone,
                                       ids);
    }

    /**
     * get calendars
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find calendars
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Calendar[] getCalendars(String   username    ,
                                   String   credentials ,
                                   String[] ids         ,
                                   String   exchangeFolder   )
        throws DataAccessException {

         return this.cd.getCalendars(username    ,
                                     credentials ,
                                     ids         ,
                                     exchangeFolder   );

    }

    /**
     * get calendar by key
     *
     * @param id
     * @param username
     * @param credentials
     *
     * @return find calendar
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Calendar getCalendarById(String username    ,
                                    String credentials ,
                                    String id          ,
                                    String exchangeFolder   )
        throws DataAccessException {

        Calendar[] calendars = null;

        calendars = this.cd.getCalendars(username          ,
                                         credentials       ,
                                         new String[] {id} ,
                                         exchangeFolder);

        if (calendars == null || !(calendars.length > 0)) {
            return null;
        }

        return calendars[0];

    }

    /**
     * add calendar
     *
     * @param calendar
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new calendar
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Calendar setCalendar(Calendar calendar    ,
                                String   username    ,
                                String   credentials ,
                                String   exchangeFolder
                              )
        throws DataAccessException {

        return this.cd.setCalendar(calendar    ,
                                   username    ,
                                   credentials ,
                                   exchangeFolder   );

    }

    /**
     * remove calendar
     *
     * @param calendar
     * @param username
     * @param principal
     * @param exchangeFolder
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void removeCalendar(Calendar calendar  ,
                               String   username  ,
                               String   principal ,
                               String   exchangeFolder )
        throws DataAccessException {

        this.cd.removeCalendar(calendar, username, principal, exchangeFolder);

    }

}
