/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.common.dao;

import java.io.IOException;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;

import java.text.MessageFormat;

import java.util.ArrayList;
import java.util.Date;
import java.util.TimeZone;

import sync4j.framework.server.store.NotFoundException;

import sync4j.exchange.httptransport.WebDavHttpTransport;
import sync4j.exchange.items.calendar.dao.CalendarDAO;
import sync4j.exchange.items.common.model.Item;
import sync4j.exchange.items.contact.dao.ContactDAO;
import sync4j.exchange.AuthenticationException;
import sync4j.exchange.DataAccessException;
import sync4j.exchange.ExchangeAccessException;

import sync4j.exchange.util.DataAccess;
import sync4j.exchange.util.DateTools;

import sync4j.exchange.xml.XmlParseException;
import sync4j.exchange.xml.XmlParser;

import sync4j.exchange.util.StringTools;

/*
 * This class implement methods to access items common data
 * in database source and exchange datastore
 *
 * @version $Id: ItemDAO.java,v 1.22 2006/02/13 13:38:53 fabius Exp $
 *
 **/
public class ItemDAO {

    //--------------------------------------------------------------------- Constants
    // Define a common initial size for the StringBuffer.
    protected static final int INITIAL_SIZE_SBUFFER = 2048; // 2K

    protected static final String FILE_ENCODING = "UTF-8";

    protected static final String WEBDAV_HEADER_PROPPATCH                  =
    "PROPPATCH {0}/{1} HTTP/1.0\r\n";

    protected static final String WEBDAV_HEADER_REMOVE                     =
    "DELETE {0}/{1} HTTP/1.0\r\n";

    protected static final String WEBDAV_HEADER_SELECT                     =
    "SEARCH {0} HTTP/1.0\r\n"                                              +
    "Depth: 1,noroot\r\n";

    private static final String WEBDAV_HEADER_PROPFIND                     =
    "PROPFIND {0} HTTP/1.0\r\n"                                            +
    "Depth: 1,noroot\r\n";

    private static final String WEBDAV_HEADER_PROPFIND_ACCESS_STATUS       =
    "PROPFIND {0} HTTP/1.0\r\n"                                            +
    "Depth: 0\r\n";

    private static final String WEBDAV_MSG_ACCESS_STATUS                   =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                        +
    "<D:propfind xmlns:D=\"DAV:\">\n"                                      +
    "<D:allprop/>\n"                                                       +
    "</D:propfind>\n"                                                      ;

    private static final String WEBDAV_MSG_SELECT_ITEM                     =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                        +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                +
    "<D:sql>\n"                                                            +
    "Select "                                                              +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "          +
    "\"DAV:getlastmodified\" AS getlastmodified, "                         +
    "\"DAV:creationdate\" AS creationdate, "                               +
    "\"DAV:href\" AS href, "                                               +
    "\"DAV:isfolder\" AS isfolder "                                        +
    "FROM \"/{0}/{1}/{2}\"  "                                              +
    "{3}"                                                                  +
    "</D:sql>\n"                                                           +
    "</D:searchrequest>"                                                   ;

    private static final String WEBDAV_MSG_SELECT_CONTACT_ITEM             =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                        +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                +
    "<D:sql>\n"                                                            +
    "Select "                                                              +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "          +
    "\"DAV:getlastmodified\" AS getlastmodified, "                         +
    "\"DAV:creationdate\" AS creationdate, "                               +
    "\"DAV:href\" AS href, "                                               +
    "\"urn:schemas:contacts:givenName\" AS " + ContactDAO.TAG_FIRSTNAME    +
                                                    ", " +
    "\"urn:schemas:contacts:sn\" AS "        + ContactDAO.TAG_LASTNAME     +
                                                    ", " +
    "\"DAV:isfolder\" AS isfolder "                                        +
    "FROM \"/{0}/{1}/{2}\"  "                                              +
    "{3}"                                                                  +
    "</D:sql>\n"                                                           +
    "</D:searchrequest>"                                                   ;

    private static final String WEBDAV_MSG_SELECT_CALENDAR_ITEM            =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                        +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                +
    "<D:sql>\n"                                                            +
    "Select "                                                              +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "          +
    "\"DAV:getlastmodified\" AS getlastmodified, "                         +
    "\"DAV:creationdate\" AS creationdate, "                               +
    "\"DAV:href\" AS href, "                                               +
    "\"urn:schemas:calendar:alldayevent\" AS "
                    + CalendarDAO.TAG_ALLDAYEVENT  + ", "                  +
   "\"urn:schemas:calendar:location\" AS "
                    + CalendarDAO.TAG_LOCATION     + ", "                  +
     "\"urn:schemas:calendar:dtstart\" AS "
                    + CalendarDAO.TAG_DATESTART    + ", "                  +
    "\"urn:schemas:calendar:dtend\" AS "
                    + CalendarDAO.TAG_DATEEND      + ", "                  +
    "\"DAV:isfolder\" AS isfolder "                                        +
    "FROM \"/{0}/{1}/{2}\"  "                                              +
    "{3}"                                                                  +
    "</D:sql>\n"                                                           +
    "</D:searchrequest>"                                                   ;

    private static final String WEBDAV_MSG_SELECT_NOTE_ITEM                =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                        +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                +
    "<D:sql>\n"                                                            +
    "Select "                                                              +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "          +
    "\"DAV:getlastmodified\" AS getlastmodified, "                         +
    "\"DAV:creationdate\" AS creationdate, "                               +
    "\"DAV:href\" AS href, "                                               +
    "\"urn:schemas:httpmail:subject\" AS subject, "                        +
    "\"DAV:isfolder\" AS isfolder "                                        +
    "FROM \"/{0}/{1}/{2}\"  "                                              +
    "{3}"                                                                  +
    "</D:sql>\n"                                                           +
    "</D:searchrequest>"                                                   ;

    private static final String WEBDAV_MSG_SELECT_TASK_ITEM                =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                        +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                +
    "<D:sql>\n"                                                            +
    "Select "                                                              +
    "\"urn:schemas:httpmail:subject\" AS subject, "                        +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8104\" as date, "               +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8105\" as duedate, "            +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "          +
    "\"DAV:getlastmodified\" AS getlastmodified, "                         +
    "\"DAV:creationdate\" AS creationdate, "                               +
    "\"DAV:href\" AS href, "                                               +
    "\"DAV:isfolder\" AS isfolder "                                        +
    "FROM \"/{0}/{1}/{2}\"  "                                              +
    "{3}"                                                                  +
    "</D:sql>\n"                                                           +
    "</D:searchrequest>"                                                   ;

    private static final String WEBDAV_MSG_PROPFIND_SELECT_ITEM            =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                        +
    "<D:propfind "                                                         +
    "xmlns:D=\"DAV:\" "                                                    +
    "xmlns:EN=\"http://schemas.microsoft.com/repl/\""                      +
    ">\n"                                                                  +
    "<D:prop><D:href/></D:prop>\n"                                         +
    "<D:prop><D:creationdate/></D:prop>\n"                                 +
    "<D:prop><D:getlastmodified/></D:prop>\n"                              +
    "<D:prop><EN:repl-uid/></D:prop>\n"                                    +
    "</D:propfind>\n"                                                      ;

    private static final String CLAUSE_REPLID_FIELD
        = "\"http://schemas.microsoft.com/repl/repl-uid\""          ;

    private static final String CONTENT_REPLID_START   = "rid:"     ;

    private static final String SQL_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE    =
    "select id, href, last_modified, twin_crc "                            +
    "from ss_exch_item_status "                                            +
    "where "                                                               +
    "uri = ? and principal = ?"                                            ;

    private static final String SQL_SELECT_HREF_BY_BY_ID_PRINCIPAL_BY_SOURCE =
    "select href "                                                         +
    "from ss_exch_item_status "                                            +
    "where "                                                               +
    "uri = ? and principal = ? and id = ?"                                 ;

    private static final String SQL_GET_TWIN                               =
     "select id, href, last_modified, twin_crc "                           +
     "from ss_exch_item_status "                                           +
     "where "                                                              +
     "uri = ? and principal = ? and twin_crc = ?"                          ;

    private static final String SQL_ADD_LOCAL_ITEM                         =
    "insert into ss_exch_item_status "                                     +
    "(id, href, last_modified, uri, principal, twin_crc) "                 +
    "values(?, ?, ?, ?, ?, ?)"                                             ;

    private static final String SQL_UPDATE_LOCAL_ITEM                      =
    "update ss_exch_item_status "                                          +
    "set last_modified = ? , twin_crc = ? "                                +
    "where id = ? and uri = ? and principal = ?"                           ;

    private static final String SQL_DELETE_LOCAL_ITEM                      =
    "delete from ss_exch_item_status "                                     +
    "where id = ? and uri = ? and principal = ? "                          ;

    private static final String TAG_HREF             = "href"              ;
    private static final String TAG_CREATION_DATE    = "creationdate"      ;
    private static final String TAG_REPLUID          = "repluid"           ;
    private static final String TAG_LAST_MODIFIED    = "getlastmodified"   ;
    protected
            static final String TAG_IS_FOLDER        = "isfolder"          ;
    private static final String TAG_PROP             = "a:prop"            ;

    protected
        static final String PROP_NO_FOLDER           = "0"                 ;

    protected static final String NEW_LINE           = "\r\n"              ;

    private static final int
        STATUS_ACCESS_DANIED                         = 401                 ;
    private static final int
        STATUS_OK_LOAD_CHANGE                        = 207                 ;
    private static final int
        STATUS_OK_DELETE                             = 200                 ;
    private static final int
        STATUS_NOT_FOUND                             = 404                 ;

    public static final String TAG_RESPONSE           = "a:response"       ;

    //--------------------------------------------------------------------- Private data

    private DataAccess           da                  = null ;
    private WebDavHttpTransport  webDavHttp          = null ;

    //--------------------------------------------------------------------- Costructors

    public ItemDAO ()
        throws DataAccessException {

        this.da = new DataAccess ()  ;

    }

    public ItemDAO (String  host ,
                    int     port )
    throws DataAccessException {

        this.da          = new DataAccess ()  ;
        this.webDavHttp  = new WebDavHttpTransport(host, port);

    }

    //--------------------------------------------------------------------- Public methods

    /**
     * get local items
     *
     * @param sourceURI
     * @param principal
     *
     * @return array of find local items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item[] getLocalItems (String sourceURI, int principal)
    throws DataAccessException {

         Connection         connection     = null ;
         PreparedStatement  ps             = null ;
         ResultSet          rs             = null ;

         ArrayList          rows           = null ;

         Item[]             items          = null ;
         Item               item           = null ;

         int                count          = 0    ;

         try {

             connection = this.da.getConnection();

             rows = new ArrayList();

             ps = connection.prepareStatement(
                    SQL_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE);

             ps.setString (1, sourceURI ) ;
             ps.setInt    (2, principal ) ;

             rs = ps.executeQuery();

             while(rs.next()) {
                 rows.add(new Item(rs.getString(1),
                                 rs.getString(2),
                                 new Date(rs.getTimestamp(3).getTime()),
                                 rs.getLong(4)));
                 ++count;
             }

             items = (Item[]) rows.toArray(new Item[count]);

         } catch (Exception e) {
             throw new DataAccessException("Error getting local items", e);
         } finally {
             try {
                 this.da.cleanUp(connection, ps, rs);
             } catch (Exception e)  {
                 throw new DataAccessException("Error getting local items", e);
             }
         }

         return items;

    }

     /**
      * get href
      *
      * @param id
      * @param principal
      * @param sourceURI
      *
      * @return href
      *
      * @throws sync4j.exchange.util.DataAccessException
      **/
     public String getHref (String id, int principal, String sourceURI)
     throws DataAccessException {

          Connection         connection     = null ;
          PreparedStatement  ps             = null ;
          ResultSet          rs             = null ;

          String             href           = null ;

          try {

              connection = this.da.getConnection();

              ps = connection.prepareStatement(
                     SQL_SELECT_HREF_BY_BY_ID_PRINCIPAL_BY_SOURCE);

              ps.setString  (1, sourceURI ) ;
              ps.setInt     (2, principal ) ;
              ps.setString  (3, id        ) ;


              rs = ps.executeQuery();

              while(rs.next()) {
                  href = rs.getString(1);
              }

          } catch (Exception e) {
              throw new DataAccessException("Error getting href", e);
          } finally {
              try {
                  this.da.cleanUp(connection, ps, rs);
              } catch (Exception e)  {
                  throw new DataAccessException("Error getting href", e);
              }
          }

         return href;

     }


    /**
     * add local items
     *
     * @param items
     * @param sourceURI
     * @param principal
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void addItems(Item[] items     ,
                         String sourceURI ,
                         int    principal)
    throws DataAccessException {

         Connection         connection    = null ;
         PreparedStatement  ps            = null ;

         String             id            = null ;
         String             href          = null ;
         Date               lastModified  = null ;

         try {

            connection = this.da.getConnection();

            for (int i=0, l = items.length; i < l; i++) {

                ps = connection.prepareStatement(SQL_ADD_LOCAL_ITEM);

                id            = items[i].getId()           ;
                href          = items[i].getHref()         ;
                lastModified  = items[i].getLastModified() ;

                ps.setString    (1, id                                    ) ;
                ps.setString    (2, href                                  ) ;
                ps.setTimestamp (3, new Timestamp(lastModified.getTime()) ) ;

                ps.setString    (4, sourceURI                             ) ;
                ps.setInt       (5, principal                             ) ;

                ps.setLong      (6, items[i].getCRC()                     ) ;

                ps.executeUpdate();

                ps.close() ;
                ps = null  ;

            }

         } catch (Exception e) {
              throw new DataAccessException("Error adding local items", e);
         } finally {
             try {
                 this.da.cleanUp(connection, ps, null);
             } catch (Exception e)  {
                  throw new DataAccessException("Error adding local items", e);
             }
         }

    }

     /**
      * add local items
      *
      * @param items
      * @param sourceURI
      * @param principal
      *
      * @throws sync4j.exchange.util.DataAccessException
      **/
     public void updateItems(Item[] items     ,
                             String sourceURI ,
                             int    principal)
     throws DataAccessException {

          Connection         connection    = null ;
          PreparedStatement  ps            = null ;

          String             id            = null;
          Date               lastModified  = null;

          try {

             connection = this.da.getConnection();

             for (int i=0, l = items.length; i < l; i++) {

                 ps = connection.prepareStatement(SQL_UPDATE_LOCAL_ITEM);

                 id            = items[i].getId()           ;
                 lastModified  = items[i].getLastModified() ;

                 ps.setTimestamp (1, new Timestamp(lastModified.getTime()) ) ;
                 ps.setLong      (2, items[i].getCRC()                     ) ;
                 ps.setString    (3, id                                    ) ;
                 ps.setString    (4, sourceURI                             ) ;
                 ps.setInt       (5, principal                             ) ;

                 ps.executeUpdate();

                 ps.close() ;
                 ps = null  ;

             }

          } catch (Exception e) {
              throw new DataAccessException("Error updating local items", e);
          } finally {
              try {
                  this.da.cleanUp(connection, ps, null);
              } catch (Exception e)  {
                  throw new DataAccessException("Error adding local items", e);
              }
          }

    }

    /**
     * delete local items
     *
     * @param sourceURI
     * @param principal
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void deleteItems(Item[] items, String sourceURI, int principal)
    throws DataAccessException {

         Connection         connection  = null ;
         PreparedStatement  ps          = null ;

         String             id          = null ;

         try {

             connection = this.da.getConnection();

             for (int i=0, l = items.length; i < l; i++) {

                   ps = connection.prepareStatement(SQL_DELETE_LOCAL_ITEM);

                   id            = items[i].getId()           ;

                   ps.setString    (1, id         ) ;
                   ps.setString    (2, sourceURI  ) ;
                   ps.setInt       (3, principal  ) ;

                   ps.executeUpdate();

                   ps.close() ;
                   ps = null  ;

             }

         } catch (Exception e) {
             throw new DataAccessException("Error deleting local items", e);
         } finally {
             try {
                 this.da.cleanUp(connection, ps, null);
             } catch (Exception e)  {
                 throw new DataAccessException("Error deleting local items", e);
             }
         }

    }

    /**
     * get all sourceURI type items from Exchange Server
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item[] getAllExchangeItems(String username       ,
                                      String credentials    ,
                                      String exchangeFolder )
    throws DataAccessException {

        ArrayList items                = null ;
        Item      item                 = null ;

        String[]  props                = null ;
        String    prop                 = null ;
        String[]  resp                 = null ;

        String    replUid              = null ;
        String    uri                  = null ;
        String    creationDate         = null ;
        String    lastUpdate           = null ;

        String    isFolder             = null ;

        String    response             = null ;

        String    webDavBodyMsg        = null ;

        String    webDavHeaderMsg      = null ;

        String    server               = null ;
        String    resource             = null ;

        int       count                = 0    ;

        String    clause               = null ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        if ("calendar".equals(resource.toLowerCase())) {
            clause = " WHERE not (\"http://schemas.microsoft.com/mapi/id/{00062002-0000-0000-C000-000000000046}/0x8231\" != cast(\"0\" as int) " +
                 "and (\"urn:schemas:calendar:instancetype\" != 1 and \"urn:schemas:calendar:instancetype\" != 0)) " ;
        } else {
            clause = "";
        }

        webDavBodyMsg =
                    MessageFormat.format(WEBDAV_MSG_SELECT_ITEM,
            new Object[] {server, username, resource, clause});

        webDavHeaderMsg =
            MessageFormat.format(WEBDAV_HEADER_SELECT,

            new Object[] {
                    "/"                  +
                        server           +
                        "/"              +
                        username         +
                        "/"              +
                        resource
            }
        );


        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg ,
                                                   credentials     ,
                                                   webDavBodyMsg   ,
                                                   FILE_ENCODING   );
        } catch (IOException e) {
            throw new DataAccessException("Error getting Exchange server response",
                                           e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                    +
                                          username                   +
                                          " URI "                    +
                                          exchangeFolder             +
                                          " reading exchange items"  ,
                                          e)                         ;
        }

        resp = new String[] {response};

        try {
            props = XmlParser.getXMLTag(resp, TAG_PROP);
        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing item", e);
        }

        items = new ArrayList();

        for (int i=0, l = props.length; i < l; i++) {

            try {

                prop = props[i];

                isFolder      = XmlParser.getXMLInitTagValue
                                 (prop, TAG_IS_FOLDER)        ;

                replUid       = XmlParser.getXMLTagValue
                                 (prop, TAG_REPLUID)          ;

                uri           = XmlParser.getXMLTagValue
                                 (prop, TAG_HREF)             ;

                creationDate  = XmlParser.getXMLInitTagValue
                                 (prop, TAG_CREATION_DATE)    ;

                lastUpdate    = XmlParser.getXMLInitTagValue
                                 (prop, TAG_LAST_MODIFIED)    ;

                item = new Item(getIdFromReplUid(replUid)               ,
                                XmlParser.getHrefFromUri(uri)           ,
                                DateTools.webDavTagToDate(creationDate) ,
                                DateTools.webDavTagToDate(lastUpdate));

                if(PROP_NO_FOLDER.equals(isFolder)) {
                    items.add(item);
                    count ++;
                }

            } catch (XmlParseException e) {
                throw new DataAccessException("Error parsing item", e);
            }

        }

        return (Item[])items.toArray(new Item[count]);
    }

     /**
      * get all sourceURI calendar items from Exchange Server
      *
      * @param username
      * @param credentials
      * @param exchangeFolder
      * @param timezone
      * @param ids
      *
      * @return array of find items
      *
      * @throws sync4j.exchange.util.DataAccessException
      **/
     public Item[] getTwinCalendarItems(String username       ,
                                        String credentials    ,
                                        String exchangeFolder ,
                                        TimeZone timezone     ,
                                        String[] ids)
     throws DataAccessException {

         ArrayList items                = null ;
         Item      item                 = null ;

         String[]  props                = null ;
         String    prop                 = null ;
         String[]  resp                 = null ;

         String    replUid              = null ;
         String    uri                  = null ;
         String    creationDate         = null ;
         String    lastUpdate           = null ;

         String    dateStart            = null ;
         String    dateEnd              = null ;
         String    allDay               = null ;
         String    location             = null ;
         String    dtStart              = null ;
         String    dtEnd                = null ;

         boolean   isAllDayEvent        = false ;

         String    isFolder             = null ;

         String    response             = null ;

         String    webDavBodyMsg        = null ;

         String    webDavHeaderMsg      = null ;

         String    server               = null ;
         String    resource             = null ;

         int       count                = 0    ;

         String    clause               = null ;

         server   = getServerFromExchangeFolder   (exchangeFolder );
         resource = getResourceFromExchangeFolder (exchangeFolder );

         clause = getClause(ids);

         if (clause == null || clause.length() == 0 ) {
             clause = " WHERE ";
         } else {
             clause = clause + " AND ";
         }

         clause = clause +
                  "not (\"http://schemas.microsoft.com/mapi/id/{00062002-0000-0000-C000-000000000046}/0x8231\" != cast(\"0\" as int) " +
                  "and (\"urn:schemas:calendar:instancetype\" != 1 and \"urn:schemas:calendar:instancetype\" != 0)) " ;

         webDavBodyMsg =
                     MessageFormat.format(WEBDAV_MSG_SELECT_CALENDAR_ITEM,
             new Object[] {server, username, resource, clause});

         webDavHeaderMsg =
             MessageFormat.format(WEBDAV_HEADER_SELECT,

             new Object[] {
                     "/"                  +
                         server           +
                         "/"              +
                         username         +
                         "/"              +
                         resource
             }
         );


         try {
             response = this.webDavHttp.sendRequest(webDavHeaderMsg ,
                                                    credentials     ,
                                                    webDavBodyMsg   ,
                                                    FILE_ENCODING   );
         } catch (IOException e) {
             throw new DataAccessException("Error getting Exchange server response",
                                            e);
         }

         try {
             int s = getStatusFromResponse(response);
             checkResponseStatus(s);
         } catch (Exception e) {

             throw new DataAccessException("USER "                    +
                                           username                   +
                                           " URI "                    +
                                           exchangeFolder             +
                                           " reading exchange items"  ,
                                           e)                         ;
         }

         resp = new String[] {response};

         try {
             props = XmlParser.getXMLTag(resp, TAG_RESPONSE);
         } catch (XmlParseException e) {
             throw new DataAccessException("Error parsing item", e);
         }

         items = new ArrayList();

         for (int i=0, l = props.length; i < l; i++) {

             try {

                prop = props[i];

                isFolder      = XmlParser.getXMLInitTagValue
                                (prop, TAG_IS_FOLDER)        ;

                replUid       = XmlParser.getXMLTagValue
                                (prop, TAG_REPLUID)          ;

                uri           = XmlParser.getXMLTagValue
                                (prop, TAG_HREF)             ;

                creationDate  = XmlParser.getXMLInitTagValue
                                (prop, TAG_CREATION_DATE)    ;

                lastUpdate    = XmlParser.getXMLInitTagValue
                                (prop, TAG_LAST_MODIFIED)    ;

                dateStart     = XmlParser.getXMLInitTagValue
                                (prop, CalendarDAO.TAG_DATESTART)   ;

                dateEnd       = XmlParser.getXMLInitTagValue
                                (prop, CalendarDAO.TAG_DATEEND)     ;

                allDay        = XmlParser.getXMLInitTagValue
                                (prop, CalendarDAO.TAG_ALLDAYEVENT) ;

                location      = XmlParser.getXMLTagValue
                                (prop, CalendarDAO.TAG_LOCATION)    ;

                if (allDay != null) {
                    if("1".equals(allDay)) {
                        isAllDayEvent = true;
                    } else if("0".equals(allDay)) {
                        isAllDayEvent = false;
                    }
                }

                if (!isAllDayEvent) {
                    dtStart = DateTools.webDavTagToPDIDate(dateStart);
                    dtEnd   = DateTools.webDavTagToPDIDate(dateEnd);

                } else {
                    //
                    // We have to convert the date in yyyy-MM-dd
                    //
                    dtStart = DateTools.convertWebDavDateInDayFormat(dateStart, timezone, true);
                    dtEnd   = DateTools.convertWebDavDateInDayFormat(dateEnd, timezone, false);
                }

                item = new Item(getIdFromReplUid(replUid)            ,
                             XmlParser.getHrefFromUri(uri)           ,
                             DateTools.webDavTagToDate(creationDate) ,
                             DateTools.webDavTagToDate(lastUpdate)   ,
                             StringTools.getCRCFromString(dtStart + dtEnd + location));

                if(PROP_NO_FOLDER.equals(isFolder)) {
                    items.add(item);
                    count ++;
                }

             } catch (XmlParseException e) {
                 throw new DataAccessException("Error parsing item", e);
             }

         }

         return (Item[])items.toArray(new Item[count]);
    }


    /**
     * get all sourceURI contact items from Exchange Server
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     * @param ids
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item[] getTwinContactItems(String username       ,
                                      String credentials    ,
                                      String exchangeFolder ,
                                      String[] ids)
    throws DataAccessException {

        ArrayList items                = null ;
        Item      item                 = null ;

        String[]  props                = null ;
        String    prop                 = null ;
        String[]  resp                 = null ;

        String    replUid              = null ;
        String    uri                  = null ;
        String    creationDate         = null ;
        String    lastUpdate           = null ;

        String    firstName            = null ;
        String    lastName             = null ;

        String    isFolder             = null ;

        String    response             = null ;

        String    webDavBodyMsg        = null ;

        String    webDavHeaderMsg      = null ;

        String    server               = null ;
        String    resource             = null ;

        int       count                = 0    ;

        String    clause               = null ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        clause = getClause(ids);

        webDavBodyMsg =
                    MessageFormat.format(WEBDAV_MSG_SELECT_CONTACT_ITEM,
            new Object[] {server, username, resource, clause});

        webDavHeaderMsg =
            MessageFormat.format(WEBDAV_HEADER_SELECT,

            new Object[] {
                    "/"                  +
                        server           +
                        "/"              +
                        username         +
                        "/"              +
                        resource
            }
        );


        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg ,
                                                   credentials     ,
                                                   webDavBodyMsg   ,
                                                   FILE_ENCODING   );
        } catch (IOException e) {
            throw new DataAccessException("Error getting Exchange server response",
                                           e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                    +
                                          username                   +
                                          " URI "                    +
                                          exchangeFolder             +
                                          " reading exchange items"  ,
                                          e)                         ;
        }

        resp = new String[] {response};

        try {
            props = XmlParser.getXMLTag(resp, TAG_RESPONSE);
        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing item", e);
        }

        items = new ArrayList();

        for (int i=0, l = props.length; i < l; i++) {

            try {

                prop = props[i];

                isFolder      = XmlParser.getXMLInitTagValue
                                 (prop, TAG_IS_FOLDER)        ;

                replUid       = XmlParser.getXMLTagValue
                                 (prop, TAG_REPLUID)          ;

                uri           = XmlParser.getXMLTagValue
                                 (prop, TAG_HREF)             ;

                creationDate  = XmlParser.getXMLInitTagValue
                                 (prop, TAG_CREATION_DATE)    ;

                lastUpdate    = XmlParser.getXMLInitTagValue
                                 (prop, TAG_LAST_MODIFIED)    ;


                firstName = XmlParser.getXMLInitTagValue (
                                prop,
                                ContactDAO.TAG_FIRSTNAME
                            );

                lastName  = XmlParser.getXMLInitTagValue (
                                prop,
                                ContactDAO.TAG_LASTNAME
                            );

                item = new Item(getIdFromReplUid(replUid)            ,
                             XmlParser.getHrefFromUri(uri)           ,
                             DateTools.webDavTagToDate(creationDate) ,
                             DateTools.webDavTagToDate(lastUpdate)   ,
                             StringTools.getCRCFromString(firstName + lastName));


                if(PROP_NO_FOLDER.equals(isFolder)) {
                    items.add(item);
                    count ++;
                }

            } catch (XmlParseException e) {
                throw new DataAccessException("Error parsing item", e);
            }

        }

        return (Item[])items.toArray(new Item[count]);
    }


   /**
     * get all sourceURI note items from Exchange Server
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item[] getTwinNoteItems(String username       ,
                                   String credentials    ,
                                   String exchangeFolder ,
                                   String[] ids          )
    throws DataAccessException {

        ArrayList items                = null ;
        Item      item                 = null ;

        String[]  props                = null ;
        String    prop                 = null ;
        String[]  resp                 = null ;

        String    replUid              = null ;
        String    uri                  = null ;
        String    creationDate         = null ;
        String    lastUpdate           = null ;

        String    subject              = null ;

        String    isFolder             = null ;

        String    response             = null ;

        String    webDavBodyMsg        = null ;

        String    webDavHeaderMsg      = null ;

        String    server               = null ;
        String    resource             = null ;

        int       count                = 0    ;

        String    clause               = null ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        clause = getClause(ids);

        webDavBodyMsg =
                    MessageFormat.format(WEBDAV_MSG_SELECT_NOTE_ITEM,
            new Object[] {server, username, resource, clause});

        webDavHeaderMsg =
            MessageFormat.format(WEBDAV_HEADER_SELECT,

            new Object[] {
                    "/"                  +
                        server           +
                        "/"              +
                        username         +
                        "/"              +
                        resource
            }
        );


        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg ,
                                                   credentials     ,
                                                   webDavBodyMsg   ,
                                                   FILE_ENCODING   );
        } catch (IOException e) {
            throw new DataAccessException("Error getting Exchange server response",
                                           e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                    +
                                          username                   +
                                          " URI "                    +
                                          exchangeFolder             +
                                          " reading exchange items"  ,
                                          e)                         ;
        }

        resp = new String[] {response};

        try {
            props = XmlParser.getXMLTag(resp, TAG_RESPONSE);
        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing item", e);
        }

        items = new ArrayList();

        for (int i=0, l = props.length; i < l; i++) {

            try {

                prop = props[i];

                isFolder      = XmlParser.getXMLInitTagValue
                                 (prop, TAG_IS_FOLDER)        ;

                replUid       = XmlParser.getXMLTagValue
                                 (prop, TAG_REPLUID)          ;

                uri           = XmlParser.getXMLTagValue
                                 (prop, TAG_HREF)             ;

                creationDate  = XmlParser.getXMLInitTagValue
                                 (prop, TAG_CREATION_DATE)    ;

                lastUpdate    = XmlParser.getXMLInitTagValue
                                 (prop, TAG_LAST_MODIFIED)    ;


                subject       = XmlParser.getXMLInitTagValue
                                 (prop, "subject")            ;

                item = new Item(getIdFromReplUid(replUid)            ,
                             XmlParser.getHrefFromUri(uri)           ,
                             DateTools.webDavTagToDate(creationDate) ,
                             DateTools.webDavTagToDate(lastUpdate)   ,
                             StringTools.getCRCFromString(subject));

                if(PROP_NO_FOLDER.equals(isFolder)) {
                    items.add(item);
                    count ++;
                }

            } catch (XmlParseException e) {
                throw new DataAccessException("Error parsing item", e);
            }

        }

        return (Item[])items.toArray(new Item[count]);
    }

    /**
     * get all sourceURI note items from Exchange Server
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     * @param ids
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item[] getTwinTaskItems(String username       ,
                                   String credentials    ,
                                   String exchangeFolder ,
                                   String[] ids          )
    throws DataAccessException {

        ArrayList items                = null ;
        Item      item                 = null ;

        String[]  props                = null ;
        String    prop                 = null ;
        String[]  resp                 = null ;

        String    replUid              = null ;
        String    uri                  = null ;
        String    creationDate         = null ;
        String    lastUpdate           = null ;

        String    subject              = null ;
        String    date                 = null ;
        String    dueDate              = null ;

        String    isFolder             = null ;

        String    response             = null ;

        String    webDavBodyMsg        = null ;

        String    webDavHeaderMsg      = null ;

        String    server               = null ;
        String    resource             = null ;

        int       count                = 0    ;

        String    clause               = null ;


        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        clause = getClause(ids);

        webDavBodyMsg =
                    MessageFormat.format(WEBDAV_MSG_SELECT_TASK_ITEM,
            new Object[] {server, username, resource, clause});

        webDavHeaderMsg =
            MessageFormat.format(WEBDAV_HEADER_SELECT,

            new Object[] {
                    "/"                  +
                        server           +
                        "/"              +
                        username         +
                        "/"              +
                        resource
            }
        );


        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg ,
                                                   credentials     ,
                                                   webDavBodyMsg   ,
                                                   FILE_ENCODING   );
        } catch (IOException e) {
            throw new DataAccessException("Error getting Exchange server response",
                                           e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                    +
                                          username                   +
                                          " URI "                    +
                                          exchangeFolder             +
                                          " reading exchange items"  ,
                                          e)                         ;
        }

        resp = new String[] {response};

        try {
            props = XmlParser.getXMLTag(resp, TAG_RESPONSE);
        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing item", e);
        }

        items = new ArrayList();

        for (int i=0, l = props.length; i < l; i++) {

            try {

                prop = props[i];

                isFolder      = XmlParser.getXMLInitTagValue
                                 (prop, TAG_IS_FOLDER)        ;

                replUid       = XmlParser.getXMLTagValue
                                 (prop, TAG_REPLUID)          ;

                uri           = XmlParser.getXMLTagValue
                                 (prop, TAG_HREF)             ;

                creationDate  = XmlParser.getXMLInitTagValue
                                 (prop, TAG_CREATION_DATE)    ;

                lastUpdate    = XmlParser.getXMLInitTagValue
                                 (prop, TAG_LAST_MODIFIED)    ;

                subject       = XmlParser.getXMLInitTagValue
                                 (prop, "subject")            ;

                String value = XmlParser.getXMLInitTagValue
                                   (prop, "date");

                if (value != null && value.length() > 0) {
                    date = DateTools.webDavTagToPDIDate(value);
                } else {
                    date = "";
                }

                value = XmlParser.getXMLInitTagValue
                              (prop, "duedate");

                if (value != null && value.length() > 0) {
                    dueDate = DateTools.webDavTagToPDIDate(value);
                } else {
                    dueDate = "";
                }

                item = new Item(getIdFromReplUid(replUid)            ,
                             XmlParser.getHrefFromUri(uri)           ,
                             DateTools.webDavTagToDate(creationDate) ,
                             DateTools.webDavTagToDate(lastUpdate)   ,
                             StringTools.getCRCFromString(subject + date + dueDate));

                if(PROP_NO_FOLDER.equals(isFolder)) {
                    items.add(item);
                    count ++;
                }

            } catch (XmlParseException e) {
                throw new DataAccessException("Error parsing item", e);
            }

        }

        return (Item[])items.toArray(new Item[count]);
    }


    /**
     * get twin
     *
     * @param crc
     * @param principalId
     * @param sourceURI
     *
     * @return twin
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item getTwin(long crc        ,
                        int principalId ,
                        String sourceURI)
    throws DataAccessException {

         Connection         connection     = null ;
         PreparedStatement  ps             = null ;
         ResultSet          rs             = null ;

         ArrayList          rows           = null ;

         Item               item           = null ;

         int                count          = 0    ;

         try {

             connection = this.da.getConnection();

             rows = new ArrayList();

             ps = connection.prepareStatement(
                    SQL_GET_TWIN);

             ps.setString (1, sourceURI   ) ;
             ps.setInt    (2, principalId ) ;
             ps.setLong   (3, crc         ) ;

             rs = ps.executeQuery();

             while(rs.next()) {
                 item = new Item(rs.getString(1),
                                 rs.getString(2),
                                 new Date(rs.getTimestamp(3).getTime()),
                                 rs.getLong(4));
             }

         } catch (Exception e) {
             throw new DataAccessException("Error getting twin local item", e);
         } finally {
             try {
                 this.da.cleanUp(connection, ps, rs);
             } catch (Exception e)  {
                 throw new DataAccessException("Error getting twin local item", e);
             }
         }

         return item;

    }


    /**
     * get all sourceURI type items from Exchange Server
     * @param exchangeServerName
     * @param userName
     * @param credentials
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public int getExchangeAccessStatus(String exchangeServerName  ,
                                       String userName            ,
                                       String credentials         )
    throws DataAccessException {


        String webDavHeaderMsg = null ;
        String msg             = null ;

        int    status          = 0    ;

        webDavHeaderMsg
            = MessageFormat.format(ItemDAO.WEBDAV_HEADER_PROPFIND_ACCESS_STATUS,
                new Object[] {
                        "/"                 +
                        exchangeServerName  +
                        "/"                 +
                        userName            +
                        "/"
              }
        );

        try {
            msg = this.webDavHttp.sendRequest (webDavHeaderMsg,
                                               credentials,
                                               WEBDAV_MSG_ACCESS_STATUS,
                                               FILE_ENCODING);
        } catch (IOException e) {
            throw new DataAccessException("Error getting Exchange server response",
                                           e);
        }


        try {
            status = getStatusFromResponse(msg);
        } catch (Exception e) {

            throw new DataAccessException("USER "                    +
                                          userName                   +
                                          " access exchange items"   ,
                                          e)                         ;
        }

        return status;

    }

    /**
     * Build a clause
     * to webdav select request
     * between ids item array
     *
     * @param ids
     * @return webdav select clause
     **/
    public static String getClause (String[] ids) {

        String clause = null;

        int l = ids.length;

        if (l == 0) {
            return "";
        }

        clause = "where ("            +
                 CLAUSE_REPLID_FIELD  +
                 " = '"               +
                 CONTENT_REPLID_START +
                 ids[0]               +
                 "'"                  ;

        if (l > 1) {

            for (int i = 1; i < l; i++) {
                clause = clause       +
                " OR "                +
                CLAUSE_REPLID_FIELD   +
                " = '"                +
                CONTENT_REPLID_START  +
                ids[i]                +
                "'"                   ;
            }

        }

        clause = clause + ")\r\n";

        return clause;

    }

    /**
     * Make a id from replid webdavtag.
     *
     * @param replUid
     * @return id
     * @throws XmlParseException
     **/
    public static String getIdFromReplUid(String replUid)
    throws XmlParseException {

        int start = 0;

        start = replUid.indexOf(CONTENT_REPLID_START);

        if (start == -1) {
            throw new XmlParseException("Error in replUid-TAG, value: " + replUid);
        }

        return replUid.substring(start + CONTENT_REPLID_START.length());
    }


    //--------------------------------------------------------------------- Protected methods

    /**
     * get status from response message
     *
     * @param msg
     *
     * @throws ExchangeAccessException in case of other errors
     *
     **/
    protected int getStatusFromResponse(String msg)
    throws ExchangeAccessException
    {

        String header = null;
        String value  = null;
        int    status = 0   ;

        char[] newLine = {13, 10};

        if (msg == null && msg.length() < 30) {
            throw new ExchangeAccessException
                ("Exchange server, wrong response: " + msg);
        }

        header = msg.substring(0, msg.indexOf(new String(newLine)));

        value = header.substring(header.indexOf(" ") + 1);
        value = value.substring(0, value.indexOf(" "));

        try {
            status = Integer.parseInt(value);
        } catch  (Exception e) {
            throw new ExchangeAccessException
                ("Exchange server, wrong status: " + value);
        }

        return status;

    }


    /**
     * Check response message from Exchange server
     *
     * @param status the status to be checked
     *
     * @throws AuthenticationException in case the request has not been
     *                                 authenticated by the server
     * @throws ExchangeAccessException in case of other errors
     *
     **/
    protected void checkResponseStatus(int status)
    throws AuthenticationException ,
           ExchangeAccessException ,
           NotFoundException
    {

        if (status == STATUS_ACCESS_DANIED) {
            throw new AuthenticationException
                ("Exchange server, access denied");
        }

        if (!(status == STATUS_OK_LOAD_CHANGE ) &&
            !(status == STATUS_OK_DELETE      ) &&
            !(status == STATUS_NOT_FOUND      ) ) {
            throw new ExchangeAccessException
                ("Exchange server, error accessing resource");
        }

        if (status == STATUS_NOT_FOUND) {
            throw new NotFoundException
                ("Exchange server, resource not found");
        }

    }

    protected String getServerFromExchangeFolder(String uri) {
        return uri.substring(0, uri.indexOf("/"));
    }

    protected String getResourceFromExchangeFolder(String uri) {
        return uri.substring(uri.indexOf("/")  + 1);
    }

}
