/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.common.manager;

import java.io.IOException;

import sync4j.exchange.items.common.dao.ItemDAO;
import sync4j.exchange.items.common.model.Item;
import sync4j.exchange.DataAccessException;

/*
 * This class defines methods to access commons items data
 * in database source and exchange datastore
 *
 * @version $Id: ItemManager.java,v 1.11 2006/02/13 13:39:36 fabius Exp $
 *
 **/
public class ItemManager {

    //------------------------------------------------------------- Constants

    //------------------------------------------------------------- Private Data

    //------------------------------------------------------------- Private Data

    ItemDAO id = null ;

    //------------------------------------------------------------- Costructor

    public ItemManager ()
        throws DataAccessException {

        this.id = new ItemDAO ();

    }

    public ItemManager (String  host ,
                        int     port )
    throws DataAccessException {

        this.id = new ItemDAO (host, port);
    }

    //------------------------------------------------------------- Public methods


    /**
     * get all local items
     *
     * @param sourceURI
     * @param principal
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item[] getLocalItems (String sourceURI, int principal)
    throws DataAccessException {

         return id.getLocalItems (sourceURI, principal);

    }

     /**
      * get href
      *
      * @param itemId
      * @param principal
      * @param sourceURI
      *
      * @return href
      *
      * @throws sync4j.exchange.util.DataAccessException
      **/
     public String getHref (String itemId, int principal, String sourceURI)
     throws DataAccessException {

          return id.getHref (itemId, principal, sourceURI);

     }

    /**
     * add local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void addItems (Item[] localItems ,
                          String sourceURI  ,
                          int    principal  )
    throws DataAccessException {

         id.addItems (localItems, sourceURI, principal);

    }

    /**
     * update local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void updateItems (Item[] localItems ,
                             String sourceURI  ,
                             int principal     )
    throws DataAccessException {

         id.updateItems (localItems, sourceURI, principal);

    }


    /**
     * update local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void upgradeItems (Item[] localItems ,
                              String sourceURI  ,
                              int principal     )
    throws DataAccessException {

         id.deleteItems (localItems, sourceURI, principal);
         id.addItems    (localItems, sourceURI, principal);

    }

    /**
     * delete local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void deleteItems(Item[] localItems ,
                            String sourceURI  ,
                            int    principal  )
    throws DataAccessException {

         id.deleteItems(localItems ,
                        sourceURI  ,
                        principal  );

    }

    /**
     * get all Exchange Server items
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item[] getExchangeItems(String username       ,
                                   String credentials    ,
                                   String exchangeFolder )
    throws DataAccessException {

        return id.getAllExchangeItems(username    ,
                                      credentials ,
                                      exchangeFolder);
    }


    /**
     * get Exchange Access Status
     *
     * @param exchangeServerName
     * @param userName
     * @param credentials
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public int getExchangeAccessStatus(String exchangeServerName ,
                                       String userName           ,
                                       String credentials        )
    throws DataAccessException {

        return id.getExchangeAccessStatus(exchangeServerName ,
                                          userName           ,
                                          credentials        );
    }


}