/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.contact.manager;

import java.io.IOException;

import java.util.List;

import sync4j.exchange.items.common.dao.ItemDAO;
import sync4j.exchange.items.common.model.Item;
import sync4j.exchange.items.contact.dao.ContactDAO;
import sync4j.exchange.items.contact.model.Contact;
import sync4j.exchange.DataAccessException;

import sync4j.foundation.pdi.contact.ContactDetail;
import sync4j.foundation.pdi.contact.Email;
import sync4j.foundation.pdi.contact.Name;
import sync4j.foundation.pdi.contact.PersonalDetail;

import sync4j.exchange.util.StringTools;

/*
 * This class defines methods to access contacts data
 * in exchange server datastore
 *
 * @version $Id: ContactManager.java,v 1.13 2006/02/13 13:39:37 fabius Exp $
 *
 **/
public class ContactManager {

    //------------------------------------------------------------- Constants

    //------------------------------------------------------------- Private Data

    private ContactDAO  cd = null ;
    private ItemDAO     id = null ;

    //------------------------------------------------------------- Costructor

    public ContactManager(String exchangeServerHost ,
                          int    exchangeServerPort )
        throws DataAccessException {

        this.cd = new ContactDAO (exchangeServerHost ,
                                  exchangeServerPort);

        this.id = new ItemDAO (exchangeServerHost,
                               exchangeServerPort);

    }

    //------------------------------------------------------------- Public methods

    /**
     * get all contacts
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find contacts
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact[] getAllContacts(String   username    ,
                                    String   credentials ,
                                    String   exchangeFolder   )
        throws DataAccessException {

        return this.cd.getContacts(username      ,
                                   credentials   ,
                                   new String[0] ,
                                   exchangeFolder     );

    }

    /**
     * get contacts
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find contacts
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact[] getContacts(String   username    ,
                                 String   credentials ,
                                 String[] ids         ,
                                 String   exchangeFolder   )
        throws DataAccessException {

        return this.cd.getContacts(username    ,
                                   credentials ,
                                   ids         ,
                                   exchangeFolder   );

    }

    /**
     * get contact by key
     *
     * @param id
     * @param username
     * @param credentials
     *
     * @return find contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact getContactById(String username    ,
                                  String credentials ,
                                  String id          ,
                                  String exchangeFolder   )
        throws DataAccessException {

        Contact[] contacts = null;

        contacts = this.cd.getContacts(username          ,
                                       credentials       ,
                                       new String[] {id} ,
                                       exchangeFolder         );

        if (contacts == null || !(contacts.length > 0)) {
            return null;
        }

        return contacts[0];

    }

    /**
     * get contact twin
     *
     * @param contact the Contact object
     * @param principalId
     * @param sourceURI
     *
     * @return find contact twin
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact getContactTwin(Contact contact       ,
                                  int     principalId   ,
                                  String  sourceURI)
    throws DataAccessException {

        Name name = contact.getName () ;

        String firstName = null;
        String lastName  = null;

        if (name != null) {
            if (name.getFirstName() != null) {
                firstName   = name.getFirstName        ().
                                        getPropertyValueAsString()         ;
            }
            if (name.getLastName() != null) {
                lastName    = name.getLastName         ().
                                        getPropertyValueAsString()         ;
            }
        }

        long crc = StringTools.getCRCFromString(firstName + lastName);

        Item item = id.getTwin(crc           ,
                               principalId   ,
                               sourceURI);

        if (item == null) {
            return null;
        }

        return new Contact(item.getId(), item.getLastModified());
    }

    /**
     * get all Exchange Server twin items
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item[] getTwinContactItems(String username       ,
                                      String credentials    ,
                                      String exchangeFolder ,
                                      String[] ids          )
    throws DataAccessException {

        return id.getTwinContactItems(username    ,
                                      credentials ,
                                      exchangeFolder,
                                      ids);
    }


    /**
     * add contact
     *
     * @param contact
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact setContact(Contact contact     ,
                              String  username    ,
                              String  credentials ,
                              String  exchangeFolder   )
        throws DataAccessException {

        return this.cd.setContact(contact     ,
                                  username    ,
                                  credentials ,
                                  exchangeFolder   );

    }

    /**
     * remove contact
     *
     * @param contact
     * @param username
     * @param principal
     * @param exchangeFolder
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void removeContact(Contact contact   ,
                              String  username  ,
                              String  principal ,
                              String  exchangeFolder )
        throws DataAccessException {

        this.cd.removeContact(contact   ,
                              username  ,
                              principal ,
                              exchangeFolder );

    }

}
