/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.note.manager;

import sync4j.exchange.items.common.dao.ItemDAO;
import sync4j.exchange.items.common.model.Item;
import sync4j.exchange.items.note.dao.NoteDAO;
import sync4j.exchange.items.note.model.Note;
import sync4j.exchange.DataAccessException;

import sync4j.exchange.util.StringTools;

/*
 * This class defines methods to access notes data
 * in exchange server datastore
 *
 * @version $Id: NoteManager.java,v 1.10 2006/02/13 13:39:37 fabius Exp $
 *
 **/
public class NoteManager {

    //------------------------------------------------------------- Constants

    //------------------------------------------------------------- Private Data

    private NoteDAO nd = null ;
    private ItemDAO id = null ;

    //------------------------------------------------------------- Costructor

    public NoteManager(String exchangeServerHost ,
                       int    exchangeServerPort )
        throws DataAccessException {

        this.nd = new NoteDAO (exchangeServerHost  ,
                               exchangeServerPort );

        this.id = new ItemDAO (exchangeServerHost,
                               exchangeServerPort);

    }

    //------------------------------------------------------------- Public methods

    /**
     * get all notes
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find notes
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Note[] getAllNotes(String   username       ,
                              String   credentials    ,
                              String   exchangeFolder )
        throws DataAccessException {

        return this.nd.getNotes(username       ,
                                credentials    ,
                                new String[0]  ,
                                exchangeFolder );
    }

    /**
     * get notes
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find notes
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Note[] getNotes(String   username       ,
                           String   credentials    ,
                           String[] ids            ,
                           String   exchangeFolder )
        throws DataAccessException {

         return this.nd.getNotes(username       ,
                                 credentials    ,
                                 ids            ,
                                 exchangeFolder );

    }

    /**
     * get note by id
     *
     * @param id
     * @param username
     * @param credentials
     *
     * @return find note
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Note getNoteById(String username      ,
                            String credentials   ,
                            String id            ,
                            String exchangeFolder)
        throws DataAccessException {

        Note[] notes = null;

        notes = this.nd.getNotes(username          ,
                                 credentials       ,
                                 new String[] {id} ,
                                 exchangeFolder    );

        if (notes == null || !(notes.length > 0)) {
            return null;
        }

        return notes[0];

    }

    /**
     * add / update note
     *
     * @param note
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new note
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Note setNote(Note    note           ,
                        String  username       ,
                        String  credentials    ,
                        String  exchangeFolder )
        throws DataAccessException {

        return this.nd.setNote(note           ,
                               username       ,
                               credentials    ,
                               exchangeFolder );

    }

    /**
     * remove note
     *
     * @param note
     * @param username
     * @param principal
     * @param exchangeFolder
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void removeNote(Note     note           ,
                           String   username       ,
                           String   principal      ,
                           String   exchangeFolder )
        throws DataAccessException {

        this.nd.removeNote(note, username, principal, exchangeFolder);

    }

    /**
     * get note twin
     *
     * @param note the note object
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return find contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Note getNoteTwin(Note    note,
                            String username,
                            String credentials,
                            String exchangeFolder)
    throws DataAccessException {

        Note[] notes = null;

        String subject = null;

        try {

            subject = note.getSubject();

        } catch (Exception e) {
            throw new DataAccessException(e.getMessage());
        }

        String[] fields = {
                          NoteDAO.TAG_SUBJECT
        };

        Object[] values = {
                          subject
        };

        notes = this.nd.getNotes(username,
                                 credentials,
                                 fields,
                                 values,
                                 exchangeFolder);

        if (notes == null || !(notes.length > 0)) {
            return null;
        }

        return notes[0];
    }

    /**
     * get note twin
     *
     * @param note the Contact object
     * @param principalId
     * @param sourceURI
     *
     * @return find note twin
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Note getNoteTwin(Note    note          ,
                            int     principalId   ,
                            String  sourceURI)
    throws DataAccessException {

        long crc = StringTools.getCRCFromString(note.getSubject());

        Item item = id.getTwin(crc           ,
                               principalId   ,
                               sourceURI);

        if (item == null) {
            return null;
        }

        return new Note(item.getId(), item.getLastModified());
    }

    /**
      * get all Exchange Server twin items
      *
      * @param username
      * @param credentials
      * @param exchangeFolder
      * @param ids
      *
      * @return array of find items
      *
      * @throws sync4j.exchange.util.DataAccessException
      **/
     public Item[] getTwinNoteItems(String   username       ,
                                    String   credentials    ,
                                    String   exchangeFolder ,
                                    String[] ids            )
     throws DataAccessException {

         return id.getTwinNoteItems(username       ,
                                    credentials    ,
                                    exchangeFolder ,
                                    ids            );
     }

}
