/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.task.manager;

import java.io.IOException;

import sync4j.exchange.items.common.dao.ItemDAO;
import sync4j.exchange.items.common.model.Item;
import sync4j.exchange.items.task.dao.TaskDAO;
import sync4j.exchange.items.task.model.Task;
import sync4j.exchange.DataAccessException;
import sync4j.exchange.xml.XmlParser;
import java.util.Date;

import sync4j.exchange.util.DateTools;
import sync4j.exchange.util.StringTools;

/*
 * This class defines methods to access task data
 * in exchange server datastore
 *
 * @version $Id: TaskManager.java,v 1.11 2006/02/13 13:39:37 fabius Exp $
 *
 **/
public class TaskManager {

    //------------------------------------------------------------- Constants

    //------------------------------------------------------------- Private Data

    private TaskDAO td = null ;
    private ItemDAO id = null ;

    //------------------------------------------------------------- Costructor

    public TaskManager(String exchangeServerHost ,
                       int    exchangeServerPort)
        throws DataAccessException {

        this.td = new TaskDAO (exchangeServerHost ,
                               exchangeServerPort);

        this.id = new ItemDAO (exchangeServerHost,
                               exchangeServerPort);

    }

    //------------------------------------------------------------- Public methods

    /**
     * get all tasks
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find tasks
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task[] getAllTasks(String   username       ,
                              String   credentials    ,
                              String   exchangeFolder )
        throws DataAccessException {

        return this.td.getTasks(username       ,
                                credentials    ,
                                new String[0]  ,
                                exchangeFolder );
    }

    /**
     * get tasks
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find tasks
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task[] getTasks(String   username       ,
                           String   credentials    ,
                           String[] ids            ,
                           String   exchangeFolder )
        throws DataAccessException {

         return this.td.getTasks(username       ,
                                 credentials    ,
                                 ids            ,
                                 exchangeFolder );

    }

    /**
     * get task by id
     *
     * @param id
     * @param username
     * @param credentials
     *
     * @return find task
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task getTaskById(String username       ,
                            String credentials    ,
                            String id             ,
                            String exchangeFolder )
        throws DataAccessException {

        Task[] tasks = null;

        tasks = this.td.getTasks(username          ,
                                 credentials       ,
                                 new String[] {id} ,
                                 exchangeFolder    );

        if (tasks == null || !(tasks.length > 0)) {
            return null;
        }

        return tasks[0];

    }

    /**
     * get task twin
     *
     * @param task the Task object
     * @param principalId
     * @param sourceURI
     *
     * @return find contact twin
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task getTaskTwin(Task    task          ,
                            int     principalId   ,
                            String  sourceURI)
    throws DataAccessException {

        String date    = null;
        String dueDate = null;

        try {

            if(task.getDate() != null) {
                date = DateTools.dateToPDIDate(task.getDate());
            } else {
                date = "";
            }

            if(task.getDueDate() != null) {
                dueDate = DateTools.dateToPDIDate(task.getDueDate());
            } else {
                dueDate = "";
            }

        } catch (Exception e) {
            throw new DataAccessException (e.getMessage());
        }

        long crc = StringTools.getCRCFromString(task.getSubject() +
                                                date              +
                                                dueDate           );
        Item item = id.getTwin(crc           ,
                               principalId   ,
                               sourceURI);

        if (item == null) {
            return null;
        }

        return new Task(item.getId(), item.getLastModified());
    }

    /**
     * add / update task
     *
     * @param task
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new tsk
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task setTask(Task    task           ,
                        String  username       ,
                        String  credentials    ,
                        String  exchangeFolder )
        throws DataAccessException {

        return this.td.setTask(task           ,
                               username       ,
                               credentials    ,
                               exchangeFolder );

    }

    /**
     * remove task
     *
     * @param task
     * @param username
     * @param principal
     * @param exchangeFolder
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void removeTask(Task     task           ,
                           String   username       ,
                           String   principal      ,
                           String   exchangeFolder )
        throws DataAccessException {

        this.td.removeTask(task, username, principal, exchangeFolder);

    }

    /**
      * get all Exchange Server twin items
      *
      * @param username
      * @param credentials
      * @param exchangeFolder
      * @param ids
      *
      * @return array of find items
      *
      * @throws sync4j.exchange.util.DataAccessException
      **/
     public Item[] getTwinTaskItems(String   username       ,
                                    String   credentials    ,
                                    String   exchangeFolder ,
                                    String[] ids)
     throws DataAccessException {

         return id.getTwinTaskItems(username    ,
                                    credentials ,
                                    exchangeFolder,
                                    ids);
     }

}
