/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package sync4j.exchange.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.TimeZone;

import sync4j.foundation.pdi.utils.TimeUtils;

import sync4j.exchange.xml.XmlParseException;


/**
 * Utility class that groups dates manipulation functions.
 *
 * @version $Id: DateTools.java,v 1.5 2006/02/07 09:55:26 luigiafassina Exp $
 */
public class DateTools {

    // ---------------------------------------------------------------- Contants
    private static final String DATE_FORMAT_WEBDAV =
            "yyyy-MM-dd'T'HH:mm:ss'.000Z'";

    private static final String DATE_FORMAT_PDI =
            "yyyyMMdd'T'HHmmss'Z'";

    private static final String ANNIVERSARY_FORMAT_PDI =
            "yyyy-MM-dd";

    private static final String ANNIVERSARY_FORMAT_WEBDAV =
            "yyyy-MM-dd'T'HH:mm:ss'.000Z'";

    private static final String DATETIME_FORMAT_CLIENT_UTC =
            "yyyyMMdd'T'HHmmss'Z'";

    private static final String DATETIME_FORMAT_CLIENT_NO_UTC =
            "yyyyMMdd'T'HHmmss";

    private static final String DATE_FORMAT_WEBDAV_FROM_EXCHANGE     =
            "yyyy-MM-dd HH:mm:ss.SSS"                                ;

    protected static final int MAX_YEAR = 4000;


    /**
     * Make a webdav tag date from a date
     *
     * @param date date
     * @return webdav tag date
     * @throws XmlParseException
     **/
    public static String dateToWebDavTag(String date) throws Exception {

        SimpleDateFormat formatter = null;
        String webDavDate = null;
        Date dt = null;

        if (date != null) {

            formatter = new SimpleDateFormat(DATE_FORMAT_PDI);

            dt = formatter.parse(date);

            webDavDate = dateToWebDavTag(dt);

        } else {
            webDavDate = "";
        }

        return webDavDate;
    }

    /**
     * Make a webdav tag date from a date
     *
     * @param date date
     * @return webdav tag date
     * @throws XmlParseException
     **/
    public static String dateToWebDavTag(Date date) {

        SimpleDateFormat formatter = null;
        String webDavDate = null;

        if (date != null) {

            formatter = new SimpleDateFormat(DATE_FORMAT_WEBDAV);

            webDavDate = formatter.format(date);

            if (webDavDate.length() > 4) {
                String year = webDavDate.substring(0, 4);
                if (Integer.parseInt(year) > MAX_YEAR) {
                    webDavDate = "";
                }
            }
        } else {
            webDavDate = "";
        }

        return webDavDate;
    }


    /**
     * Make a webdav tag date from a date in day format.
     *
     * @param date date
     * @param tz timezone
     * @return webdav tag date
     * @throws XmlParseException
     **/
    public static String dateInDayFormatToWebDavTag(String date,
            TimeZone tz,
            boolean isAStartDate) throws Exception {

        if (isAStartDate) {
            //
            // The start date has time 00:00:00
            //
            date = TimeUtils.convertDateFromInDayFormat(date, "000000", false);
        } else {
            //
            // The start date has time 23:59:00
            //
            date = TimeUtils.convertDateFromInDayFormat(date, "240000", false);
        }

        date = TimeUtils.convertLocalDateToUTC(date, tz);

        return dateToWebDavTag(date);
    }


    /**
     * Make a date from a webdav tag date
     *
     * @param date webdav tag content
     * @return a Date
     * @throws XmlParseException
     **/
    public static Date pdiToDate(String date) throws XmlParseException {

        SimpleDateFormat formatter = null;
        Date dt = null;

        try {

            if (date != null) {

                formatter = new SimpleDateFormat(DATE_FORMAT_PDI);

                dt = formatter.parse(date);
            } else {
                dt = null;
            }
        } catch (ParseException e) {
            throw new XmlParseException(e.toString());
        }

        return dt;

    }

    /**
     * Make a date from a webdav tag date
     *
     * @param date webdav tag content
     * @return a Date
     * @throws XmlParseException
     **/
    public static String webDavToPDIAnniversary(String date) throws Exception {

        SimpleDateFormat formatter = null;

        Date dt = null;
        String pdiDate = null;

        if (date != null && date.length() > 0) {

            formatter = new SimpleDateFormat(ANNIVERSARY_FORMAT_WEBDAV);

            dt = formatter.parse(date);

            formatter = new SimpleDateFormat(ANNIVERSARY_FORMAT_PDI);

            pdiDate = formatter.format(dt);

        } else {
            pdiDate = null;
        }

        return pdiDate;

    }

    /**
     * Make a date from a webdav tag date
     *
     * @param date webdav tag content
     * @return a Date
     * @throws XmlParseException
     **/
    public static String PDIToWebDavAnniversary(String date) throws Exception {

        SimpleDateFormat formatter = null;

        Date dt = null;
        String pdiDate = null;

        if (date != null && date.length() > 0) {

            formatter = new SimpleDateFormat(ANNIVERSARY_FORMAT_PDI);

            dt = formatter.parse(date);

            formatter = new SimpleDateFormat(ANNIVERSARY_FORMAT_WEBDAV);

            pdiDate = formatter.format(dt);

        } else {
            pdiDate = null;
        }

        return pdiDate;

    }

    /**
     * Make a date from a webdav tag date
     *
     * @param date webdav tag content
     * @return a Date
     * @throws XmlParseException
     **/
    public static Date webDavTagToDate (String date)
    throws XmlParseException {

        SimpleDateFormat  formatter    = null ;
        String            value        = null ;
        Date              timestamp    = null ;

        try {

            if (date != null) {
                value = date.substring(0,  10) +
                        " "                    +
                        date.substring(11, 23) ;

                formatter = new SimpleDateFormat(DATE_FORMAT_WEBDAV_FROM_EXCHANGE);

                timestamp = formatter.parse(value);
            } else {
                timestamp = null;
            }
        } catch (ParseException e) {
            throw new XmlParseException(e.toString());
        }

        return timestamp;

    }

    /**
     * Make a date from a webdav tag date to client date
     *
     * @param date webdav tag content
     * @return a Date
     * @throws XmlParseException
     **/
    public static String  toClientDate (Date date)
    throws XmlParseException {

        SimpleDateFormat  formatter    = null ;
        String            value        = null ;

        try {

            if (date != null) {

                formatter = new SimpleDateFormat(DATETIME_FORMAT_CLIENT_UTC);

                return formatter.format(date);

            } else {
                return null;
            }
        } catch (Exception e) {
            throw new XmlParseException(e.toString());
        }

    }

    /**
     * Make a date from a client date to webdav date
     *
     * @param date client date
     * @return a Date
     * @throws XmlParseException
     **/
    public static Date clientDateToDate (String date)
    throws XmlParseException {

        SimpleDateFormat  formatter    = null ;
        Date              timestamp    = null ;

        String            format       = null ;

        try {

            if (date != null) {

                if (date.indexOf("Z") != -1) {
                    format = DATETIME_FORMAT_CLIENT_UTC;
                } else {
                    //
                    // for nokia phone utc bug
                    //
                    format = DATETIME_FORMAT_CLIENT_NO_UTC;
                }

                formatter = new SimpleDateFormat(format);

                timestamp = formatter.parse(date);

                return timestamp;

            } else {
                return null;
            }
        } catch (ParseException e) {
            throw new XmlParseException(e.toString());
        }
    }

    /**
     * Make a date from a webdav tag date
     *
     * @param date webdav tag content
     * @return a Date
     * @throws XmlParseException
     **/
    public static String webDavTagToPDIDate (String date)
    throws XmlParseException {

        SimpleDateFormat  formatter    = null ;

        Date              dt           = null ;
        String            pdiDate      = null ;

        if (date != null && date.length() > 0) {

            dt = webDavTagToDate (date);

            formatter = new SimpleDateFormat(DATE_FORMAT_PDI);

            pdiDate = formatter.format(dt);

        } else {
            pdiDate = null;
        }

        return pdiDate;
    }


    /**
     * Make a date from a webdav tag date
     *
     * @param date webdav tag content
     * @return a Date
     * @throws XmlParseException
     **/
    public static String dateToPDIDate (Date date)
    throws XmlParseException {

        SimpleDateFormat  formatter    = null ;

        String pdiDate = null;

        if (date != null ) {

            formatter = new SimpleDateFormat(DATE_FORMAT_PDI);

            pdiDate = formatter.format(date);

        } else {
            pdiDate = null;
        }

        return pdiDate;
    }



    public static String convertWebDavDateInDayFormat(
            String date,
            TimeZone timezone,
            boolean isAStartDate) throws XmlParseException {

        SimpleDateFormat  formatter    = null ;

        Date              dt           = null ;
        String            dayDate      = null ;

        if (date != null && date.length() > 0) {

           String            value        = null ;

           try {

               value = date.substring(0, 10) +
                       " " +
                       date.substring(11, 23);

               formatter = new SimpleDateFormat(DATE_FORMAT_WEBDAV_FROM_EXCHANGE);
               formatter.setTimeZone(TimeZone.getTimeZone("UTC"));
               dt = formatter.parse(value);

               //
               // If it isn't a startDate, we have to substract one day
               // (27/10/2005 00:00:00 --> 26/10/2005 23:59:59). In order to do
               // it quickly, we substract one millisecond
               //
               if (!isAStartDate) {
                   dt.setTime(dt.getTime() - 1);
               }

               if (timezone != null) {
                   formatter.setTimeZone(timezone);
               }
               formatter.applyPattern(ANNIVERSARY_FORMAT_PDI);
               dayDate = formatter.format(dt);

           } catch (ParseException e) {
               throw new XmlParseException(e.toString());
           }

        } else {
            dayDate = null;
        }

        return dayDate;
    }


    public static void main(String[] args) throws Exception {
        String dateStart = "2005-09-26T22:00:00.000Z";
        String dateEnd   = "2005-09-27T22:00:00.000Z";
        TimeZone tz = TimeZone.getTimeZone("Europe/Rome");
        dateStart = convertWebDavDateInDayFormat(dateStart, tz, true);
        dateEnd = convertWebDavDateInDayFormat(dateEnd, tz, false);

        System.out.println("Date start: " + dateStart);
        System.out.println("Date end: " + dateEnd);
    }
}
